@@ -8,7 +8,8 @@ import java.io.File;
  * a common base directory.
  * @since 1.0-alpha-9
  */
-public interface FileSet extends BaseFileSet
+public interface FileSet
+    extends BaseFileSet
 {
     /**
      * Returns the file sets base directory.
