@@ -24,11 +24,8 @@ import java.io.FileInputStream;
 import java.io.FileOutputStream;
 import java.io.IOException;
 import java.io.InputStream;
-import java.io.InputStreamReader;
 import java.io.OutputStreamWriter;
 import java.io.PrintWriter;
-import java.io.Reader;
-import java.io.UnsupportedEncodingException;
 import java.util.ArrayList;
 import java.util.Collections;
 import java.util.Comparator;
@@ -36,22 +33,27 @@ import java.util.Enumeration;
 import java.util.HashSet;
 import java.util.Iterator;
 import java.util.List;
+import java.util.Set;
+import java.util.SortedMap;
 import java.util.StringTokenizer;
 import java.util.TreeMap;
 import java.util.Vector;
-
 import org.codehaus.plexus.archiver.ArchiverException;
 import org.codehaus.plexus.archiver.util.EnumeratedAttribute;
 import org.codehaus.plexus.archiver.zip.ZipArchiver;
 import org.codehaus.plexus.archiver.zip.ZipEntry;
 import org.codehaus.plexus.archiver.zip.ZipFile;
 import org.codehaus.plexus.archiver.zip.ZipOutputStream;
+import org.codehaus.plexus.logging.Logger;
+import org.codehaus.plexus.logging.console.ConsoleLogger;
+import org.codehaus.plexus.util.IOUtil;
 
 /**
  * Base class for tasks that build archives in JAR file format.
  *
  * @version $Revision$ $Date$
  */
+@SuppressWarnings( { "NullableProblems" } )
 public class JarArchiver
     extends ZipArchiver
 {
@@ -108,11 +110,6 @@ public class JarArchiver
      */
     private Manifest manifest;
 
-    /**
-     * The encoding to use when reading in a manifest file
-     */
-    private String manifestEncoding;
-
     /**
      * The file found from the 'manifest' attribute.  This can be
      * either the location of a manifest, or the name of a jar added
@@ -139,12 +136,12 @@ public class JarArchiver
      * <p/>
      * Will not be filled unless the user has asked for an index.
      */
-    private Vector rootEntries;
+    private Vector<String> rootEntries;
 
     /**
      * Path containing jars that shall be indexed in addition to this archive.
      */
-    private ArrayList indexJars;
+    private ArrayList<String> indexJars;
 
     /**
      * constructor
@@ -154,33 +151,32 @@ public class JarArchiver
         super();
         archiveType = "jar";
         setEncoding( "UTF8" );
-        rootEntries = new Vector();
+        rootEntries = new Vector<String>();
     }
 
     /**
      * Set whether or not to create an index list for classes.
      * This may speed up classloading in some cases.
+     *
+     * @param flag true to create an index
      */
     public void setIndex( boolean flag )
     {
         index = flag;
     }
 
-    /**
-     * Set whether or not to create an index list for classes.
-     * This may speed up classloading in some cases.
-     */
+    @SuppressWarnings( { "JavaDoc", "UnusedDeclaration" } )
+    @Deprecated // Useless method. Manifests should be UTF-8 by convention. Calling this setter does nothing
     public void setManifestEncoding( String manifestEncoding )
     {
-        this.manifestEncoding = manifestEncoding;
     }
 
     /**
      * Allows the manifest for the archive file to be provided inline
      * in the build file rather than in an external file.
      *
-     * @param newManifest
-     * @throws ManifestException
+     * @param newManifest The new manifest
+     * @throws ManifestException .
      */
     public void addConfiguredManifest( Manifest newManifest )
         throws ManifestException
@@ -191,25 +187,26 @@ public class JarArchiver
         }
         else
         {
-            configuredManifest.merge( newManifest );
+            JdkManifestFactory.merge( configuredManifest, newManifest, false );
         }
         savedConfiguredManifest = configuredManifest;
     }
 
     /**
-     * The manifest file to use. This can be either the location of a manifest,
-     * or the name of a jar added through a fileset. If its the name of an added
-     * jar, the task expects the manifest to be in the jar at META-INF/MANIFEST.MF.
+     * The manifest file to use. This can be either the location of a manifest, or the name of a jar added through a
+     * fileset. If its the name of an added jar, the task expects the manifest to be in the jar at META-INF/MANIFEST.MF.
      *
      * @param manifestFile the manifest file to use.
+     * @throws org.codehaus.plexus.archiver.ArchiverException
+     *          .
      */
+    @SuppressWarnings( { "UnusedDeclaration" } )
     public void setManifest( File manifestFile )
         throws ArchiverException
     {
         if ( !manifestFile.exists() )
         {
-            throw new ArchiverException( "Manifest file: " + manifestFile
-                                         + " does not exist." );
+            throw new ArchiverException( "Manifest file: " + manifestFile + " does not exist." );
         }
 
         this.manifestFile = manifestFile;
@@ -218,70 +215,34 @@ public class JarArchiver
     private Manifest getManifest( File manifestFile )
         throws ArchiverException
     {
-        Manifest newManifest = null;
-        FileInputStream fis;
-        InputStreamReader isr = null;
+        InputStream in = null;
         try
         {
-            fis = new FileInputStream( manifestFile );
-            if ( manifestEncoding == null )
-            {
-                isr = new InputStreamReader( fis );
-            }
-            else
-            {
-                isr = new InputStreamReader( fis, manifestEncoding );
-            }
-            newManifest = getManifest( isr );
-        }
-        catch ( UnsupportedEncodingException e )
-        {
-            throw new ArchiverException( "Unsupported encoding while reading manifest: "
-                                         + e.getMessage(), e );
+            in = new FileInputStream( manifestFile );
+            return getManifest( in );
         }
         catch ( IOException e )
         {
-            throw new ArchiverException( "Unable to read manifest file: "
-                                         + manifestFile
-                                         + " (" + e.getMessage() + ")", e );
+            throw new ArchiverException( "Unable to read manifest file: " + manifestFile + " (" + e.getMessage() + ")",
+                                         e );
         }
         finally
         {
-            if ( isr != null )
-            {
-                try
-                {
-                    isr.close();
-                }
-                catch ( IOException e )
-                {
-                    // do nothing
-                }
-            }
+            IOUtil.close( in );
         }
-        return newManifest;
     }
 
-    private Manifest getManifest( Reader r )
+    private Manifest getManifest( InputStream is )
         throws ArchiverException
     {
-        Manifest newManifest;
         try
         {
-            newManifest = new Manifest( r );
-        }
-        catch ( ManifestException e )
-        {
-            getLogger().error( "Manifest is invalid: " + e.getMessage() );
-            throw new ArchiverException( "Invalid Manifest: " + manifestFile,
-                                         e );
+            return new Manifest( is );
         }
         catch ( IOException e )
         {
-            throw new ArchiverException( "Unable to read manifest file"
-                                         + " (" + e.getMessage() + ")", e );
+            throw new ArchiverException( "Unable to read manifest file" + " (" + e.getMessage() + ")", e );
         }
-        return newManifest;
     }
 
     /**
@@ -297,13 +258,13 @@ public class JarArchiver
      *
      * @param config setting for found manifest behavior.
      */
+    @SuppressWarnings( { "UnusedDeclaration" } )
     public void setFilesetmanifest( FilesetManifestConfig config )
     {
         filesetManifestConfig = config;
         mergeManifestsMain = "merge".equals( config.getValue() );
 
-        if ( ( filesetManifestConfig != null )
-             && !filesetManifestConfig.getValue().equals( "skip" ) )
+        if ( ( filesetManifestConfig != null ) && !filesetManifestConfig.getValue().equals( "skip" ) )
         {
 
             doubleFilePass = true;
@@ -311,25 +272,13 @@ public class JarArchiver
     }
 
     /**
-     * Adds a zipfileset to include in the META-INF directory.
-     *
-     * @param fs zipfileset to add
-     */
-/*    public void addMetainf(ZipFileSet fs) {
-        // We just set the prefix for this fileset, and pass it up.
-        fs.setPrefix("META-INF/");
-        super.addFileset(fs);
-    }
-*/
-
-    /**
-     *
+     * @param indexJar The indexjar
      */
     public void addConfiguredIndexJars( File indexJar )
     {
         if ( indexJars == null )
         {
-            indexJars = new ArrayList();
+            indexJars = new ArrayList<String>();
         }
         indexJars.add( indexJar.getAbsolutePath() );
     }
@@ -337,7 +286,6 @@ public class JarArchiver
     protected void initZipOutputStream( ZipOutputStream zOut )
         throws IOException, ArchiverException
     {
-
         if ( !skipWriting )
         {
             Manifest jarManifest = createManifest();
@@ -350,57 +298,45 @@ public class JarArchiver
         getLogger().debug( "\n\n\nChecking for jar manifest virtual files...\n\n\n" );
         System.out.flush();
 
-        return ( configuredManifest != null ) || ( manifest != null ) || ( manifestFile != null ) || super.hasVirtualFiles();
+        return ( configuredManifest != null ) || ( manifest != null ) || ( manifestFile != null )
+            || super.hasVirtualFiles();
     }
 
     private Manifest createManifest()
         throws ArchiverException
     {
-        try
-        {
             Manifest finalManifest = Manifest.getDefaultManifest();
 
-            if ( manifest == null )
+            if ( ( manifest == null ) && ( manifestFile != null ) )
             {
-                if ( manifestFile != null )
-                {
-                    // if we haven't got the manifest yet, attempt to
-                    // get it now and have manifest be the final merge
-                    manifest = getManifest( manifestFile );
-                }
+                // if we haven't got the manifest yet, attempt to
+                // get it now and have manifest be the final merge
+                manifest = getManifest( manifestFile );
             }
 
-            /*
-             * Precedence: manifestFile wins over inline manifest,
-             * over manifests read from the filesets over the original
-             * manifest.
-             *
-             * merge with null argument is a no-op
-             */
+        /*
+        * Precedence: manifestFile wins over inline manifest,
+        * over manifests read from the filesets over the original
+        * manifest.
+        *
+        * merge with null argument is a no-op
+        */
 
-            if ( isInUpdateMode() )
-            {
-                finalManifest.merge( originalManifest );
-            }
-            finalManifest.merge( filesetManifest );
-            finalManifest.merge( configuredManifest );
-            finalManifest.merge( manifest, !mergeManifestsMain );
-
-            return finalManifest;
-
-        }
-        catch ( ManifestException e )
+        if ( isInUpdateMode() )
         {
-            getLogger().error( "Manifest is invalid: " + e.getMessage() );
-            throw new ArchiverException( "Invalid Manifest", e );
+            JdkManifestFactory.merge( finalManifest, originalManifest, false );
         }
+        JdkManifestFactory.merge( finalManifest, filesetManifest, false );
+        JdkManifestFactory.merge( finalManifest, configuredManifest, false );
+        JdkManifestFactory.merge( finalManifest, manifest, !mergeManifestsMain );
+
+        return finalManifest;
     }
 
     private void writeManifest( ZipOutputStream zOut, Manifest manifest )
         throws IOException, ArchiverException
     {
-        for ( Enumeration e = manifest.getWarnings();
-              e.hasMoreElements(); )
+        for ( Enumeration e = manifest.getWarnings(); e.hasMoreElements(); )
         {
             getLogger().warn( "Manifest warning: " + e.nextElement() );
         }
@@ -408,23 +344,16 @@ public class JarArchiver
         zipDir( null, zOut, "META-INF/", DEFAULT_DIR_MODE );
         // time to write the manifest
         ByteArrayOutputStream baos = new ByteArrayOutputStream();
-        OutputStreamWriter osw = new OutputStreamWriter( baos, "UTF-8" );
-        PrintWriter writer = new PrintWriter( osw );
-        manifest.write( writer );
-        writer.flush();
+        manifest.write( baos );
 
-        ByteArrayInputStream bais =
-            new ByteArrayInputStream( baos.toByteArray() );
-        super.zipFile( bais, zOut, MANIFEST_NAME,
-                       System.currentTimeMillis(), null,
-                       DEFAULT_FILE_MODE );
+        ByteArrayInputStream bais = new ByteArrayInputStream( baos.toByteArray() );
+        super.zipFile( bais, zOut, MANIFEST_NAME, System.currentTimeMillis(), null, DEFAULT_FILE_MODE );
         super.initZipOutputStream( zOut );
     }
 
     protected void finalizeZipOutputStream( ZipOutputStream zOut )
         throws IOException, ArchiverException
     {
-
         if ( index )
         {
             createIndexList( zOut );
@@ -440,14 +369,15 @@ public class JarArchiver
      * @param zOut the zip stream representing the jar being built.
      * @throws IOException thrown if there is an error while creating the
      *                     index and adding it to the zip stream.
+     * @throws org.codehaus.plexus.archiver.ArchiverException
+     *                     .
      */
     private void createIndexList( ZipOutputStream zOut )
         throws IOException, ArchiverException
     {
         ByteArrayOutputStream baos = new ByteArrayOutputStream();
         // encoding must be UTF8 as specified in the specs.
-        PrintWriter writer = new PrintWriter( new OutputStreamWriter( baos,
-                                                                      "UTF8" ) );
+        PrintWriter writer = new PrintWriter( new OutputStreamWriter( baos, "UTF8" ) );
 
         // version-info blankline
         writer.println( "JarIndex-Version: 1.0" );
@@ -458,54 +388,51 @@ public class JarArchiver
 
         // filter out META-INF if it doesn't contain anything other than the index and manifest.
         // this is what sun.misc.JarIndex does, guess we ought to be consistent.
-        HashSet filteredDirs = new HashSet(addedDirs.keySet());
+        Set<String> filteredDirs = new HashSet<String>( addedDirs.keySet() );
         // our added dirs always have a trailing slash
-        if(filteredDirs.contains(META_INF_NAME+"/")) {
+        if ( filteredDirs.contains( META_INF_NAME + '/' ) )
+        {
             boolean add = false;
-            Iterator i = entries.keySet().iterator();
-            while(i.hasNext()) {
-                String entry = (String)i.next();
-                if(entry.startsWith(META_INF_NAME+"/") &&
-                        !entry.equals(INDEX_NAME) && !entry.equals(MANIFEST_NAME)) {
+            for ( String entry : entries.keySet() )
+            {
+                if ( entry.startsWith( META_INF_NAME + '/' ) && !entry.equals( INDEX_NAME ) && !entry.equals(
+                    MANIFEST_NAME ) )
+                {
                     add = true;
                     break;
                 }
             }
-            if(!add)
+            if ( !add )
             {
-                filteredDirs.remove(META_INF_NAME+"/");
+                filteredDirs.remove( META_INF_NAME + '/' );
             }
         }
-        writeIndexLikeList( new ArrayList( filteredDirs ),
-                            rootEntries, writer );
+        writeIndexLikeList( new ArrayList<String>( filteredDirs ), rootEntries, writer );
         writer.println();
 
         if ( indexJars != null )
         {
-            Manifest mf = createManifest();
-            Manifest.Attribute classpath =
-                mf.getMainSection().getAttribute( Manifest.ATTRIBUTE_CLASSPATH );
+            java.util.jar.Manifest mf = createManifest();
+            String classpath = mf.getMainAttributes().getValue( ManifestConstants.ATTRIBUTE_CLASSPATH );
             String[] cpEntries = null;
             if ( classpath != null )
             {
-                StringTokenizer tok = new StringTokenizer( classpath.getValue(),
-                                                           " " );
+                StringTokenizer tok = new StringTokenizer( classpath, " " );
                 cpEntries = new String[tok.countTokens()];
                 int c = 0;
                 while ( tok.hasMoreTokens() )
                 {
-                    cpEntries[ c++ ] = tok.nextToken();
+                    cpEntries[c++] = tok.nextToken();
                 }
             }
 
-            for ( Iterator i = indexJars.iterator(); i.hasNext(); )
+            for ( String indexJar : indexJars )
             {
-                String indexJar = (String)i.next();
                 String name = findJarName( indexJar, cpEntries );
                 if ( name != null )
                 {
-                    ArrayList dirs = new ArrayList();
-                    ArrayList files = new ArrayList();
+                    ArrayList<String> dirs = new ArrayList<String>();
+                    ArrayList<String> files = new ArrayList<String>();
                     grabFilesAndDirs( indexJar, dirs, files );
                     if ( dirs.size() + files.size() > 0 )
                     {
@@ -518,17 +445,17 @@ public class JarArchiver
         }
 
         writer.flush();
-        ByteArrayInputStream bais =
-            new ByteArrayInputStream( baos.toByteArray() );
-        super.zipFile( bais, zOut, INDEX_NAME, System.currentTimeMillis(), null,
-                       DEFAULT_FILE_MODE );
+
+        ByteArrayInputStream bais = new ByteArrayInputStream( baos.toByteArray() );
+
+        super.zipFile( bais, zOut, INDEX_NAME, System.currentTimeMillis(), null, DEFAULT_FILE_MODE );
     }
 
     /**
      * Overridden from Zip class to deal with manifests and index lists.
      */
-    protected void zipFile( InputStream is, ZipOutputStream zOut, String vPath,
-                            long lastModified, File fromArchive, int mode )
+    protected void zipFile( InputStream is, ZipOutputStream zOut, String vPath, long lastModified, File fromArchive,
+                            int mode )
         throws IOException, ArchiverException
     {
         if ( MANIFEST_NAME.equalsIgnoreCase( vPath ) )
@@ -540,13 +467,12 @@ public class JarArchiver
         }
         else if ( INDEX_NAME.equalsIgnoreCase( vPath ) && index )
         {
-            getLogger().warn( "Warning: selected " + archiveType
-                              + " files include a META-INF/INDEX.LIST which will"
-                              + " be replaced by a newly generated one." );
+            getLogger().warn( "Warning: selected " + archiveType + " files include a META-INF/INDEX.LIST which will"
+                                  + " be replaced by a newly generated one." );
         }
         else
         {
-            if ( index && ( vPath.indexOf( "/" ) == -1 ) )
+            if ( index && ( !vPath.contains( "/" ) ) )
             {
                 rootEntries.addElement( vPath );
             }
@@ -562,157 +488,41 @@ public class JarArchiver
             // If this is the same name specified in 'manifest', this
             // is the manifest to use
             getLogger().debug( "Found manifest " + file );
-            try
+            if ( is != null )
             {
-                if ( is != null )
-                {
-                    InputStreamReader isr;
-                    if ( manifestEncoding == null )
-                    {
-                        isr = new InputStreamReader( is );
-                    }
-                    else
-                    {
-                        isr = new InputStreamReader( is, manifestEncoding );
-                    }
-                    manifest = getManifest( isr );
-                }
-                else
-                {
-                    manifest = getManifest( file );
-                }
+                manifest = getManifest( is );
             }
-            catch ( UnsupportedEncodingException e )
+            else
             {
-                throw new ArchiverException( "Unsupported encoding while reading "
-                                             + "manifest: " + e.getMessage(), e );
+                manifest = getManifest( file );
             }
         }
-        else if ( ( filesetManifestConfig != null )
-                  && !filesetManifestConfig.getValue().equals( "skip" ) )
+        else if ( ( filesetManifestConfig != null ) && !filesetManifestConfig.getValue().equals( "skip" ) )
         {
             // we add this to our group of fileset manifests
             getLogger().debug( "Found manifest to merge in file " + file );
 
-            try
+            Manifest newManifest;
+            if ( is != null )
             {
-                Manifest newManifest;
-                if ( is != null )
-                {
-                    InputStreamReader isr;
-                    if ( manifestEncoding == null )
-                    {
-                        isr = new InputStreamReader( is );
-                    }
-                    else
-                    {
-                        isr = new InputStreamReader( is, manifestEncoding );
-                    }
-                    newManifest = getManifest( isr );
-                }
-                else
-                {
-                    newManifest = getManifest( file );
-                }
-
-                if ( filesetManifest == null )
-                {
-                    filesetManifest = newManifest;
-                }
-                else
-                {
-                    filesetManifest.merge( newManifest );
-                }
+                newManifest = getManifest( is );
             }
-            catch ( UnsupportedEncodingException e )
+            else
             {
-                throw new ArchiverException( "Unsupported encoding while reading "
-                                             + "manifest: " + e.getMessage(), e );
+                newManifest = getManifest( file );
             }
-            catch ( ManifestException e )
+
+            if ( filesetManifest == null )
             {
-                getLogger().error( "Manifest in file " + file + " is invalid: "
-                                   + e.getMessage() );
-                throw new ArchiverException( "Invalid Manifest", e );
+                filesetManifest = newManifest;
+            }
+            else
+            {
+                JdkManifestFactory.merge( filesetManifest, newManifest, false );
             }
-        }
-        else
-        {
-            // assuming 'skip' otherwise
-            // don't warn if skip has been requested explicitly, warn if user
-            // didn't set the attribute
-
-            // Hide warning also as it makes no sense since
-            // the filesetmanifest attribute itself has been
-            // hidden
-
-            //int logLevel = filesetManifestConfig == null ?
-            //    Project.MSG_WARN : Project.MSG_VERBOSE;
-            //log("File " + file
-            //    + " includes a META-INF/MANIFEST.MF which will be ignored. "
-            //    + "To include this file, set filesetManifest to a value other "
-            //    + "than 'skip'.", logLevel);
         }
     }
 
-    /**
-     * Collect the resources that are newer than the corresponding
-     * entries (or missing) in the original archive.
-     * <p/>
-     * <p>If we are going to recreate the archive instead of updating
-     * it, all resources should be considered as new, if a single one
-     * is.  Because of this, subclasses overriding this method must
-     * call <code>super.getResourcesToAdd</code> and indicate with the
-     * third arg if they already know that the archive is
-     * out-of-date.</p>
-     *
-     * @param filesets    The filesets to grab resources from
-     * @param zipFile     intended archive file (may or may not exist)
-     * @param needsUpdate whether we already know that the archive is
-     *                    out-of-date.  Subclasses overriding this method are supposed to
-     *                    set this value correctly in their call to
-     *                    super.getResourcesToAdd.
-     * @return an map of resources to add for each fileset passed in as well
-     *         as a flag that indicates whether the archive is uptodate.
-     * @throws ArchiverException if it likes
-     */
-/*    protected Map getResourcesToAdd(FileSet[] filesets,
-                                             File zipFile,
-                                             boolean needsUpdate)
-        throws ArchiverException {
-
-        // need to handle manifest as a special check
-        if (zipFile.exists()) {
-            // if it doesn't exist, it will get created anyway, don't
-            // bother with any up-to-date checks.
-
-            try {
-                originalManifest = getManifestFromJar(zipFile);
-                if (originalManifest == null) {
-                    getLogger().debug("Updating jar since the current jar has no manifest");
-                    needsUpdate = true;
-                } else {
-                    Manifest mf = createManifest();
-                    if (!mf.equals(originalManifest)) {
-                        getLogger().debug("Updating jar since jar manifest has changed");
-                        needsUpdate = true;
-                    }
-                }
-            } catch (Throwable t) {
-                getLogger().warn("error while reading original manifest: " + t.getMessage());
-                needsUpdate = true;
-            }
-
-        } else {
-            // no existing archive
-            needsUpdate = true;
-        }
-
-        createEmpty = needsUpdate;
-        return super.getResourcesToAdd(filesets, zipFile, needsUpdate);
-    }
-*/
-
     /**
      */
     protected boolean createEmptyZip( File zipFile )
@@ -726,8 +536,7 @@ public class JarArchiver
         ZipOutputStream zOut = null;
         try
         {
-            getLogger().debug( "Building MANIFEST-only jar: "
-                               + getDestFile().getAbsolutePath() );
+            getLogger().debug( "Building MANIFEST-only jar: " + getDestFile().getAbsolutePath() );
             zOut = new ZipOutputStream( new FileOutputStream( getDestFile() ) );
 
             zOut.setEncoding( getEncoding() );
@@ -744,22 +553,12 @@ public class JarArchiver
         }
         catch ( IOException ioe )
         {
-            throw new ArchiverException( "Could not create almost empty JAR archive"
-                                         + " (" + ioe.getMessage() + ")", ioe );
+            throw new ArchiverException( "Could not create almost empty JAR archive (" + ioe.getMessage() + ")", ioe );
         }
         finally
         {
             // Close the output stream.
-            try
-            {
-                if ( zOut != null )
-                {
-                    zOut.close();
-                }
-            }
-            catch ( IOException ex )
-            {
-            }
+            IOUtil.close( zOut );
             createEmpty = false;
         }
         return true;
@@ -806,16 +605,19 @@ public class JarArchiver
     {
         public String[] getValues()
         {
-            return new String[]{"skip", "merge", "mergewithoutmain"};
+            return new String[]{ "skip", "merge", "mergewithoutmain" };
         }
     }
 
     /**
      * Writes the directory entries from the first and the filenames
      * from the second list to the given writer, one entry per line.
+     *
+     * @param dirs   The directories
+     * @param files  The files
+     * @param writer The printwriter ;)
      */
-    protected final void writeIndexLikeList( List dirs, List files,
-                                             PrintWriter writer )
+    protected final void writeIndexLikeList( List<String> dirs, List<String> files, PrintWriter writer )
     {
         // JarIndex is sorting the directories by ascending order.
         // it has no value but cosmetic since it will be read into a
@@ -868,45 +670,47 @@ public class JarArchiver
      * <p/>
      * <p>if there is a classpath and the given file doesn't match any
      * of its entries, return null.</p>
+     *
+     * @param fileName  .
+     * @param classpath .
+     * @return The guessed name
      */
-    protected static final String findJarName( String fileName,
-                                               String[] classpath )
+    protected static String findJarName( String fileName, String[] classpath )
     {
         if ( classpath == null )
         {
-            return ( new File( fileName ) ).getName();
+            return new File( fileName ).getName();
         }
         fileName = fileName.replace( File.separatorChar, '/' );
-        TreeMap matches = new TreeMap( new Comparator()
+        SortedMap<String, String> matches = new TreeMap<String, String>( new Comparator<String>()
         {
             // longest match comes first
-            public int compare( Object o1, Object o2 )
+            public int compare( String o1, String o2 )
             {
-                if ( ( o1 instanceof String ) && ( o2 instanceof String ) )
+                if ( ( o1 != null ) && ( o2 != null ) )
                 {
-                    return ( (String) o2 ).length()
-                           - ( (String) o1 ).length();
+                    return o2.length() - o1.length();
                 }
                 return 0;
             }
         } );
 
-        for ( int i = 0; i < classpath.length; i++ )
+        for ( String aClasspath : classpath )
         {
-            if ( fileName.endsWith( classpath[ i ] ) )
+            if ( fileName.endsWith( aClasspath ) )
             {
-                matches.put( classpath[ i ], classpath[ i ] );
+                matches.put( aClasspath, aClasspath );
             }
             else
             {
-                int slash = classpath[ i ].indexOf( "/" );
-                String candidate = classpath[ i ];
+                int slash = aClasspath.indexOf( "/" );
+                String candidate = aClasspath;
                 while ( slash > -1 )
                 {
                     candidate = candidate.substring( slash + 1 );
                     if ( fileName.endsWith( candidate ) )
                     {
-                        matches.put( candidate, classpath[ i ] );
+                        matches.put( candidate, aClasspath );
                         break;
                     }
                     slash = candidate.indexOf( "/" );
@@ -914,59 +718,74 @@ public class JarArchiver
             }
         }
 
-        return matches.size() == 0
-               ? null : (String) matches.get( matches.firstKey() );
+        return matches.size() == 0 ? null : matches.get( matches.firstKey() );
     }
 
     /**
      * Grab lists of all root-level files and all directories
      * contained in the given archive.
+     *
+     * @param file  .
+     * @param files .
+     * @param dirs  .
+     * @throws java.io.IOException .
      */
-    protected static final void grabFilesAndDirs( String file, List dirs,
-                                                  List files )
+    protected static void grabFilesAndDirs( String file, List<String> dirs, List<String> files )
         throws IOException
     {
-        ZipFile zf = null;
-        try
+        File zipFile = new File( file );
+        if ( !zipFile.exists() )
         {
-            zf = new ZipFile( file, "utf-8" );
-            Enumeration entries = zf.getEntries();
-            HashSet dirSet = new HashSet();
-            while ( entries.hasMoreElements() )
+            Logger logger = new ConsoleLogger( Logger.LEVEL_INFO, "console" );
+            logger.error( "JarArchive skipping non-existing file: " + zipFile.getAbsolutePath() );
+        }
+        else if ( zipFile.isDirectory() )
+        {
+            Logger logger = new ConsoleLogger( Logger.LEVEL_INFO, "console" );
+            logger.info( "JarArchiver skipping indexJar " + zipFile + " because it is not a jar" );
+        }
+        else
+        {
+            ZipFile zf = null;
+            try
             {
-                ZipEntry ze =
-                    (ZipEntry) entries.nextElement();
-                String name = ze.getName();
-                // avoid index for manifest-only jars.
-                if (!name.equals(META_INF_NAME) && !name.equals(META_INF_NAME+"/") &&
-                        !name.equals(INDEX_NAME) && !name.equals(MANIFEST_NAME))
+                zf = new ZipFile( file, "utf-8" );
+                Enumeration<ZipEntry> entries = zf.getEntries();
+                HashSet<String> dirSet = new HashSet<String>();
+                while ( entries.hasMoreElements() )
                 {
-                    if ( ze.isDirectory() )
+                    ZipEntry ze = entries.nextElement();
+                    String name = ze.getName();
+                    // avoid index for manifest-only jars.
+                    if ( !name.equals( META_INF_NAME ) && !name.equals( META_INF_NAME + '/' ) && !name.equals(
+                        INDEX_NAME ) && !name.equals( MANIFEST_NAME ) )
                     {
-                        dirSet.add( name );
-                    }
-                    else if ( name.indexOf( "/" ) == -1 )
-                    {
-                        files.add( name );
-                    }
-                    else
-                    {
-                        // a file, not in the root
-                        // since the jar may be one without directory
-                        // entries, add the parent dir of this file as
-                        // well.
-                        dirSet.add( name.substring( 0,
-                                                    name.lastIndexOf( "/" ) + 1 ) );
+                        if ( ze.isDirectory() )
+                        {
+                            dirSet.add( name );
+                        }
+                        else if ( !name.contains( "/" ) )
+                        {
+                            files.add( name );
+                        }
+                        else
+                        {
+                            // a file, not in the root
+                            // since the jar may be one without directory
+                            // entries, add the parent dir of this file as
+                            // well.
+                            dirSet.add( name.substring( 0, name.lastIndexOf( "/" ) + 1 ) );
+                        }
                     }
                 }
+                dirs.addAll( dirSet );
             }
-            dirs.addAll( dirSet );
-        }
-        finally
-        {
-            if ( zf != null )
+            finally
             {
-                zf.close();
+                if ( zf != null )
+                {
+                    zf.close();
+                }
             }
         }
     }
