@@ -35,6 +35,7 @@ import org.codehaus.plexus.archiver.util.EnumeratedAttribute;
 import org.codehaus.plexus.archiver.util.ResourceUtils;
 import org.codehaus.plexus.components.io.attributes.PlexusIoResourceAttributes;
 import org.codehaus.plexus.components.io.resources.PlexusIoResource;
+import org.codehaus.plexus.util.IOUtil;
 import org.codehaus.plexus.util.StringUtils;
 
 /**
@@ -147,12 +148,12 @@ public class TarArchiver
     protected void execute()
         throws ArchiverException, IOException
     {
-    	if ( ! checkForced() )
-    	{
-    		return;
-    	}
+        if ( !checkForced() )
+        {
+            return;
+        }
 
-    	ResourceIterator iter = getResources();
+        ResourceIterator iter = getResources();
         if ( !iter.hasNext() )
         {
             throw new ArchiverException( "You must set at least one file." );
@@ -173,7 +174,7 @@ public class TarArchiver
             throw new ArchiverException( tarFile + " is read-only." );
         }
 
-        getLogger().info( "Building tar : " + tarFile.getAbsolutePath() );
+        getLogger().info( "Building tar: " + tarFile.getAbsolutePath() );
 
         tOut = new TarOutputStream(
             compression.compress( new BufferedOutputStream( new FileOutputStream( tarFile ) ) ) );
@@ -254,8 +255,7 @@ public class TarArchiver
                 }
                 else if ( longFileMode.isWarnMode() )
                 {
-                    getLogger().warn( "Entry: " + vPath + " longer than "
-                                      + TarConstants.NAMELEN + " characters." );
+                    getLogger().warn( "Entry: " + vPath + " longer than " + TarConstants.NAMELEN + " characters." );
                     if ( !longWarningGiven )
                     {
                         getLogger().warn( "Resulting tar file can only be processed "
@@ -272,7 +272,8 @@ public class TarArchiver
 
             TarEntry te = new TarEntry( vPath );
             long teLastModified = entry.getResource().getLastModified();
-            te.setModTime( teLastModified == PlexusIoResource.UNKNOWN_MODIFICATION_DATE ? System.currentTimeMillis() : teLastModified );
+            te.setModTime( teLastModified == PlexusIoResource.UNKNOWN_MODIFICATION_DATE ? System.currentTimeMillis()
+                            : teLastModified );
 
             if ( !entry.getResource().isDirectory() )
             {
@@ -288,10 +289,14 @@ public class TarArchiver
             
             PlexusIoResourceAttributes attributes = entry.getResourceAttributes();
             
-            te.setUserName( ( attributes != null && attributes.getUserName() != null ) ? attributes.getUserName() : options.getUserName() );
-            te.setGroupName( ( attributes != null && attributes.getGroupName() != null ) ? attributes.getGroupName() : options.getGroup() );
-            te.setUserId( ( attributes != null && attributes.getUserId() != -1 ) ? attributes.getUserId() : options.getUid() );
-            te.setGroupId( ( attributes != null && attributes.getGroupId() != -1 ) ? attributes.getGroupId() : options.getGid() );
+            te.setUserName( ( attributes != null && attributes.getUserName() != null ) ? attributes.getUserName()
+                            : options.getUserName() );
+            te.setGroupName( ( attributes != null && attributes.getGroupName() != null ) ? attributes.getGroupName()
+                            : options.getGroup() );
+            te.setUserId( ( attributes != null && attributes.getUserId() != null) ? attributes.getUserId()
+                            : options.getUid() );
+            te.setGroupId( ( attributes != null && attributes.getGroupId() != null) ? attributes.getGroupId()
+                            : options.getGid() );
 
             tOut.putNextEntry( te );
 
@@ -313,10 +318,7 @@ public class TarArchiver
         }
         finally
         {
-            if ( fIn != null )
-            {
-                fIn.close();
-            }
+            IOUtil.close( fIn );
         }
     }
 
@@ -573,22 +575,20 @@ public class TarArchiver
             {
                 return new GZIPOutputStream( ostream );
             }
-            else
+            else if ( BZIP2.equals( value ) )
             {
-                if ( BZIP2.equals( value ) )
-                {
-                    ostream.write( 'B' );
-                    ostream.write( 'Z' );
-                    return new CBZip2OutputStream( ostream );
-                }
+                ostream.write( 'B' );
+                ostream.write( 'Z' );
+                return new CBZip2OutputStream( ostream );
             }
             return ostream;
         }
     }
 
-	public boolean isSupportingForced() {
-		return true;
-	}
+    public boolean isSupportingForced()
+    {
+        return true;
+    }
 
     protected void cleanUp()
     {
@@ -599,23 +599,11 @@ public class TarArchiver
     protected void close()
         throws IOException
     {
-        if ( tOut != null )
-        {
-            try
-            {
-                // close up
-                tOut.close();
-            }
-            catch ( IOException e )
-            {
-                // ignore
-            }
-        }
+        IOUtil.close( tOut );
     }
 
     protected String getArchiveType()
     {
         return "TAR";
     }
-
 }
