@@ -17,7 +17,6 @@ package org.codehaus.plexus.archiver.zip;
  *  limitations under the License.
  */
 
-import java.io.ByteArrayInputStream;
 import java.io.ByteArrayOutputStream;
 import java.io.File;
 import java.io.FileOutputStream;
@@ -30,7 +29,6 @@ import java.util.HashMap;
 import java.util.Hashtable;
 import java.util.Map;
 import java.util.Stack;
-import java.util.Vector;
 import java.util.zip.CRC32;
 
 import org.codehaus.plexus.archiver.AbstractArchiver;
@@ -43,10 +41,12 @@ import org.codehaus.plexus.archiver.util.ResourceUtils;
 import org.codehaus.plexus.components.io.resources.PlexusIoFileResource;
 import org.codehaus.plexus.components.io.resources.PlexusIoResource;
 import org.codehaus.plexus.util.FileUtils;
+import org.codehaus.plexus.util.IOUtil;
 
 /**
  * @version $Revision$ $Date$
  */
+@SuppressWarnings({"NullableProblems", "UnusedDeclaration"})
 public abstract class AbstractZipArchiver
     extends AbstractArchiver
 {
@@ -61,6 +61,8 @@ public abstract class AbstractZipArchiver
 
     private boolean doCompress = true;
 
+    private boolean recompressAddedZips = true;
+
     private boolean doUpdate = false;
 
     // shadow of the above if the value is altered in execute
@@ -75,11 +77,9 @@ public abstract class AbstractZipArchiver
     //not used: private boolean keepCompression = false;
     private boolean doFilesonly = false;
 
-    protected Hashtable entries = new Hashtable();
+    protected Hashtable<String, String> entries = new Hashtable<String, String>();
 
-    protected Hashtable addedDirs = new Hashtable();
-
-    private Vector addedFiles = new Vector();
+    protected Hashtable<String, String> addedDirs = new Hashtable<String, String>();
 
     private static final long EMPTY_CRC = new CRC32().getValue();
 
@@ -143,6 +143,16 @@ public abstract class AbstractZipArchiver
         return doCompress;
     }
 
+    public boolean isRecompressAddedZips()
+    {
+        return recompressAddedZips;
+    }
+
+    public void setRecompressAddedZips( boolean recompressAddedZips )
+    {
+        this.recompressAddedZips = recompressAddedZips;
+    }
+
     public void setUpdateMode( boolean update )
     {
         this.doUpdate = update;
@@ -154,49 +164,10 @@ public abstract class AbstractZipArchiver
         return doUpdate;
     }
 
-//    /**
-//     * A 3 digit octal string, specify the user, group and
-//     * other modes in the standard Unix fashion;
-//     * optional, default=0644
-//     *
-//     * @deprecated use AbstractArchiver.setDefaultFileMode(int) instead.
-//     */
-//    public void setFileMode( String octalString )
-//    {
-//        setDefaultFileMode( Integer.parseInt( octalString, 8 ) );
-//    }
-//
-//    /**
-//     * @deprecated use AbstractArchiver.getDefaultFileMode() instead.
-//     */
-//    public int getFileMode()
-//    {
-//        return getDefaultFileMode();
-//    }
-//
-//    /**
-//     * A 3 digit octal string, specify the user, group and
-//     * other modes in the standard Unix fashion;
-//     * optional, default=0755
-//     *
-//     * @deprecated use AbstractArchiver.setDefaultDirectoryMode(int).
-//     */
-//    public void setDirMode( String octalString )
-//    {
-//        setDefaultDirectoryMode( Integer.parseInt( octalString, 8 ) );
-//    }
-//
-//    /**
-//     * @deprecated use AbstractArchiver.getDefaultDirectoryMode() instead.
-//     */
-//    public int getDirMode()
-//    {
-//        return getDefaultDirectoryMode();
-//    }
-//
     /**
      * If true, emulate Sun's jar utility by not adding parent directories;
      * optional, defaults to false.
+     * @param f true to emilate sun jar utility
      */
     public void setFilesonly( boolean f )
     {
@@ -219,6 +190,7 @@ public abstract class AbstractZipArchiver
      * up.  Rounding up may lead to a different type of problems like
      * JSPs inside a web archive that seem to be slightly more recent
      * than precompiled pages, rendering precompilation useless.</p>
+     * @param r true to round
      */
     public void setRoundUp( boolean r )
     {
@@ -261,8 +233,10 @@ public abstract class AbstractZipArchiver
     private void createArchiveMain()
         throws ArchiverException, IOException
     {
+        //noinspection deprecation
         if ( !Archiver.DUPLICATES_SKIP.equals( duplicate ) )
         {
+            //noinspection deprecation
             setDuplicateBehavior( duplicate );
         }
         
@@ -360,19 +334,20 @@ public abstract class AbstractZipArchiver
         success = true;
     }
 
-    protected Map getZipEntryNames( File file )
+    protected Map<String, Long> getZipEntryNames( File file )
         throws IOException
     {
         if ( !file.exists()  ||  !doUpdate )
         {
+            //noinspection unchecked
             return Collections.EMPTY_MAP;
         }
-        final Map entries = new HashMap();
+        final Map<String, Long> entries = new HashMap<String, Long>();
         final ZipFile zipFile = new ZipFile( file );
         for ( Enumeration en = zipFile.getEntries();  en.hasMoreElements();  )
         {
             ZipEntry ze = (ZipEntry) en.nextElement();
-            entries.put( ze.getName(), new Long( ze.getLastModificationTime() ) );
+            entries.put( ze.getName(), ze.getLastModificationTime());
         }
         return entries;
     }
@@ -385,11 +360,7 @@ public abstract class AbstractZipArchiver
     protected boolean isFileUpdated( ArchiveEntry entry, Map entries )
     {
         Long l = (Long) entries.get( entry.getName() );
-        if ( l == null )
-        {
-            return false;
-        }
-        return l.longValue() == -1  ||  !ResourceUtils.isUptodate( entry.getResource(), l.longValue() );
+        return l != null && (l == -1 || !ResourceUtils.isUptodate(entry.getResource(), l));
     }
 
     /**
@@ -398,6 +369,7 @@ public abstract class AbstractZipArchiver
      * @param resources the resources to add
      * @param zOut      the stream to write to
      */
+    @SuppressWarnings({"JavaDoc"})
     protected final void addResources( ResourceIterator resources, ZipOutputStream zOut )
         throws IOException, ArchiverException
     {
@@ -435,12 +407,13 @@ public abstract class AbstractZipArchiver
     /**
      * Ensure all parent dirs of a given entry have been added.
      */
+    @SuppressWarnings({"JavaDoc"})
     protected final void addParentDirs( File baseDir, String entry, ZipOutputStream zOut, String prefix )
         throws IOException
     {
         if ( !doFilesonly && getIncludeEmptyDirs() )
         {
-            Stack directories = new Stack();
+            Stack<String> directories = new Stack<String>();
 
             // Don't include the last entry itself if it's
             // a dir; it will be added on its own.
@@ -460,7 +433,7 @@ public abstract class AbstractZipArchiver
 
             while ( !directories.isEmpty() )
             {
-                String dir = (String) directories.pop();
+                String dir = directories.pop();
                 File f;
                 if ( baseDir != null )
                 {
@@ -476,6 +449,50 @@ public abstract class AbstractZipArchiver
         }
     }
 
+    private void readWithZipStats(InputStream in, byte[] header, ZipEntry ze, ByteArrayOutputStream bos) throws IOException {
+        byte[] buffer = new byte[8 * 1024];
+
+        CRC32 cal2 = new CRC32();
+
+        long size = 0;
+
+        for (byte aHeader : header) {
+            cal2.update(aHeader);
+            size++;
+        }
+
+        int count = 0;
+        do
+        {
+            size += count;
+            cal2.update( buffer, 0, count );
+            if (bos != null)
+            {
+                bos.write( buffer, 0, count );
+            }
+            count = in.read( buffer, 0, buffer.length );
+        }
+        while ( count != -1 );
+        ze.setSize(size);
+        ze.setCrc(cal2.getValue());
+    }
+
+    public static long copy( final InputStream input,
+                             final OutputStream output,
+                             final int bufferSize )
+            throws IOException
+    {
+        final byte[] buffer = new byte[bufferSize];
+        long size = 0;
+        int n;
+        while ( -1 != ( n = input.read( buffer ) ) )
+        {
+            size += n;
+            output.write( buffer, 0, n );
+        }
+        return size;
+    }
+
     /**
      * Adds a new entry to the archive, takes care of duplicates as well.
      *
@@ -487,6 +504,7 @@ public abstract class AbstractZipArchiver
      *                     entry from, will be null if we are not copying from an archive.
      * @param mode         the Unix permissions to set.
      */
+    @SuppressWarnings({"JavaDoc"})
     protected void zipFile( InputStream in, ZipOutputStream zOut, String vPath, long lastModified, File fromArchive,
                             int mode )
         throws IOException, ArchiverException
@@ -499,8 +517,17 @@ public abstract class AbstractZipArchiver
         {
             ZipEntry ze = new ZipEntry( vPath );
             ze.setTime( lastModified );
-            ze.setMethod( doCompress ? ZipEntry.DEFLATED : ZipEntry.STORED );
 
+            byte[] header = new byte[4];
+            int read = in.read(header);
+
+            boolean compressThis = doCompress;
+            if (!recompressAddedZips && isZipHeader(header)){
+                compressThis = false;
+            }
+
+            ze.setMethod( compressThis ? ZipEntry.DEFLATED : ZipEntry.STORED );
+            ze.setUnixMode( UnixStat.FILE_FLAG | mode );
             /*
              * ZipOutputStream.putNextEntry expects the ZipEntry to
              * know its size and the CRC sum before you start writing
@@ -508,66 +535,38 @@ public abstract class AbstractZipArchiver
              *
              * This forces us to process the data twice.
              */
-            if ( !zOut.isSeekable() && !doCompress )
-            {
-                long size = 0;
-                CRC32 cal = new CRC32();
-                if ( !in.markSupported() )
+
+
+
+            if (zOut.isSeekable() || compressThis) {
+                zOut.putNextEntry( ze );
+                if (read > 0) zOut.write(header, 0, read);
+                IOUtil.copy( in, zOut, 8 * 1024);
+            } else {
+                if (in.markSupported())
                 {
-                    // Store data into a byte[]
-                    ByteArrayOutputStream bos = new ByteArrayOutputStream();
-
-                    byte[] buffer = new byte[8 * 1024];
-                    int count = 0;
-
-                    do
-                    {
-                        size += count;
-                        cal.update( buffer, 0, count );
-                        bos.write( buffer, 0, count );
-                        count = in.read( buffer, 0, buffer.length );
-                    }
-                    while ( count != -1 );
-
-                    in = new ByteArrayInputStream( bos.toByteArray() );
+                    in.mark( Integer.MAX_VALUE );
+                    readWithZipStats(in, header, ze, null);
+                    in.reset();
+                    zOut.putNextEntry(ze);
+                    if (read > 0) zOut.write(header, 0, read);
+                    IOUtil.copy(in, zOut, 8 * 1024);
                 }
                 else
                 {
-                    in.mark( Integer.MAX_VALUE );
-                    byte[] buffer = new byte[8 * 1024];
-                    int count = 0;
-
-                    do
-                    {
-                        size += count;
-                        cal.update( buffer, 0, count );
-                        count = in.read( buffer, 0, buffer.length );
-                    }
-                    while ( count != -1 );
-
-                    in.reset();
+                    // Store data into a byte[]
+                    ByteArrayOutputStream bos = new ByteArrayOutputStream(128 * 1024);
+                    readWithZipStats(in, header, ze, bos);
+                    zOut.putNextEntry(ze);
+                    if (read > 0) zOut.write(header, 0, read);
+                    bos.writeTo( zOut);
                 }
-                ze.setSize( size );
-                ze.setCrc( cal.getValue() );
             }
-
-            ze.setUnixMode( UnixStat.FILE_FLAG | mode );
-            zOut.putNextEntry( ze );
-
-            byte[] buffer = new byte[8 * 1024];
-            int count = 0;
-            do
-            {
-                if ( count != 0 )
-                {
-                    zOut.write( buffer, 0, count );
-                }
-                count = in.read( buffer, 0, buffer.length );
-            }
-            while ( count != -1 );
         }
+    }
 
-        addedFiles.addElement( vPath );
+    private boolean isZipHeader(byte[] header) {
+        return header[0] == 0x50 && header[1] == 0x4b && header[2] == 03 && header[3] == 04;
     }
 
     /**
@@ -579,6 +578,7 @@ public abstract class AbstractZipArchiver
      * @param zOut  the stream to write to
      * @param vPath the name this entry shall have in the archive
      */
+    @SuppressWarnings({"JavaDoc"})
     protected void zipFile( ArchiveEntry entry, ZipOutputStream zOut, String vPath )
         throws IOException, ArchiverException
     {
@@ -587,16 +587,20 @@ public abstract class AbstractZipArchiver
             throw new ArchiverException( "A zip file cannot include itself" );
         }
 
-        InputStream fIn = entry.getInputStream();
+        InputStream in = entry.getInputStream();
         try
         {
             // ZIPs store time with a granularity of 2 seconds, round up
             final long lastModified = entry.getResource().getLastModified() + ( roundUp ? 1999 : 0 );
-            zipFile( fIn, zOut, vPath, lastModified, null, entry.getMode() );
+            zipFile( in, zOut, vPath, lastModified, null, entry.getMode() );
+        }
+        catch ( IOException e )
+        {
+            throw new ArchiverException( "IOException when zipping " + entry.getName() + ": " + e.getMessage(), e );
         }
         finally
         {
-            fIn.close();
+            IOUtil.close( in );
         }
     }
 
@@ -644,8 +648,10 @@ public abstract class AbstractZipArchiver
     /**
      * Create an empty zip file
      *
+     * @param zipFile The file
      * @return true for historic reasons
      */
+    @SuppressWarnings({"JavaDoc"})
     protected boolean createEmptyZip( File zipFile )
         throws ArchiverException
     {
@@ -672,17 +678,7 @@ public abstract class AbstractZipArchiver
         }
         finally
         {
-            if ( os != null )
-            {
-                try
-                {
-                    os.close();
-                }
-                catch ( IOException e )
-                {
-                    //ignore
-                }
-            }
+            IOUtil.close( os );
         }
         return true;
     }
@@ -705,7 +701,6 @@ public abstract class AbstractZipArchiver
     {
         super.cleanUp();
         addedDirs.clear();
-        addedFiles.removeAllElements();
         entries.clear();
         addingNewFiles = false;
         doUpdate = savedDoUpdate;
